@@ -27,13 +27,15 @@ class GRPCClient:
         secure: bool = True,
         service_name: str = None,
         timeout: int = 30,
-        conn_timeout: int = 1
+        conn_timeout: int = 1,
+        traceroute: bool = False
     ):
         self.host = host
         self.port = port
         self.metadata = (("api-key", api_key), ("service-name", service_name))
         self.secure = secure
         self.timeout = timeout
+        self.traceroute = traceroute
         self.conn_timeout = conn_timeout
         atexit.register(_clean_up_client, client=self)
 
@@ -70,6 +72,7 @@ class GRPCClient:
             path=path,
             namespace=namespace,
             timeout=self.timeout,
+            traceroute=self.traceroute
         )
 
     def get_delete_request(
@@ -87,6 +90,7 @@ class GRPCClient:
             path=path,
             namespace=namespace,
             timeout=self.timeout,
+            traceroute=self.traceroute
         )
 
     def get_fetch_request(
@@ -104,6 +108,7 @@ class GRPCClient:
             path=path,
             namespace=namespace,
             timeout=self.timeout,
+            traceroute=self.traceroute
         )
 
     def get_query_request(
@@ -139,6 +144,7 @@ class GRPCClient:
             path=path,
             namespace=namespace,
             timeout=self.timeout,
+            traceroute=self.traceroute
         )
 
     def get_info_request(
@@ -154,6 +160,7 @@ class GRPCClient:
             path=path,
             namespace=namespace,
             timeout=self.timeout,
+            traceroute=self.traceroute
         )
 
     def grpc_server_on(self) -> bool:
@@ -167,7 +174,6 @@ class GRPCClient:
         """Creates sub to send proptobuf requests."""
         stub = core_pb2_grpc.RPCClientStub(self.channel)
         if self.conn_timeout > 0 and not self.grpc_server_on():
-            # TODO: doesn't always work
             raise RuntimeError("Failed to connect to gRPC host {}:{}".format(self.host, self.port))
         for response in stub.Call(request_stream, metadata=self.metadata):
             yield response
@@ -193,4 +199,7 @@ def _generate_request_id() -> int:
 
 def _clean_up_client(client: GRPCClient):
     """Cleans up client resources"""
-    client.channel.close()
+    try:
+        client.channel.close()
+    except TypeError:
+        pass
